use std::collections::BTreeMap;
use std::sync::Arc;
use actix::prelude::*;
use actix_files as fs;
use actix_web::{middleware, web, App, HttpServer};

#[macro_use]
extern crate lazy_static;

mod websocket;
mod game;
mod datasource;
mod server {
    pub mod server;
    pub mod messages;
    pub mod gamelobby;
}

use crate::server::server::{Server, ws_initiate};
use crate::datasource::SqliteSource;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    std::env::set_var("RUST_LOG", "actix_server=info,actix_web=info");
    env_logger::init();

    let server = Server {
        lobbies: BTreeMap::new(),
        default_data: datasource::create_array_source("questions.txt")
    };
    let server_addr = server.start();

    HttpServer::new(move || {
        App::new()
            .app_data(web::Data::new(server_addr.clone()))
            // enable logger
            .wrap(middleware::Logger::default())
            // websocket route
            .service(web::resource("/ws/").route(web::get().to(ws_initiate)))
            // static files
            .service(fs::Files::new("/", "static/").index_file("index.html"))
    })
    .bind("127.0.0.1:8000")?
    .run()
    .await
}
